<?php

namespace App\Models;

use App\Command\Log;
use Dcat\Admin\Traits\HasDateTimeFormatter;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Model;

class OrderDivideRecord extends Model
{
    use HasDateTimeFormatter;
    use SoftDeletes;
    protected $table = 'order_divide_record';

    public const COMMISSION_TYPE = [
        1 => '直推佣金',//需求调整，不再分账，改成积分，支付时抵扣
        2 => '间推佣金',//需求调整，不再分账，改成积分，支付时抵扣
        3 => '用户取货佣金',
        4 => '手动调账分账',
        5 => '员工佣金',
    ];

    /**
     * 收益分配
     * @param int $order_id 订单ID
     * @param string $payconfirm_no 发起确认支付，订单流水号
     * @return array 包含分账成员和确认金额的数组
     */
    public static function divide($order_id, $payconfirm_no = '')
    {
        $div_members = []; //汇付 分账对象
        $orderObj = OrderInfo::find($order_id);
        $merchant_id = $orderObj->merchant_id; //绑定的商户
        $buyer_id = $orderObj->user_id; //下单用户ID
        $total_amount = 0; //订单商品总价
        $merchant_proportion = ''; //佣金比例
        $sp_ogid = ''; //参与分佣商品ID
        $commissionPreData = [
            'merchant_member_id' => '',
            'merchant_amount' => 0,
        ];

        //商户是否实名
        $isMerchantRealName = 0;
        if ($merchant_id) {
            $hfCompanyMObj = HfCompanyMember::where('merchant_id', $merchant_id)->first();
            $isMerchantRealName = ($hfCompanyMObj->status == 'succeeded') ? 1 : 0;
        }

        $ogList = OrderGoods::where("order_id", $order_id)->get(); //订单商品
        foreach ($ogList as $kk => $item) {
            $goods_amount = $item->goods_price * $item->goods_number;
            $total_amount += $goods_amount;
            //商品信息
            $goodObj = Good::find($item->goods_id);
            $merchant_commission = $goodObj->merchant_commission;
            $merchant_proportion .= $goodObj->merchant_commission . ",";
            $sp_ogid = $item->id . ",";

            //商户分佣记录
            if ($merchant_id && $merchant_commission >= 1 && $merchant_commission < 100) {
                $divide_price = number_format($goods_amount * ($merchant_commission / 100), 2);
                //收益直接到商户账户
                $merObj = Merchant::find($merchant_id);
                //汇付参数
                $member_id = $hfCompanyMObj->member_id;
                if ($member_id) {
                    $merObj->total_revenue += $divide_price;
                    $merObj->save();
                    $commissionPreData['merchant_member_id'] = $member_id;
                    $commissionPreData['merchant_amount'] += $divide_price;
                }
            }
        }

        //商户佣金
        if ($merchant_id && $merchant_commission >= 1 && $merchant_commission < 100 && $commissionPreData['merchant_amount'] > 0) {
            self::addRecord($sp_ogid, $order_id, $orderObj->goods_amount, $commissionPreData['merchant_amount'], $merchant_proportion, $merchant_id, $buyer_id, 3, $isMerchantRealName, $payconfirm_no);
            if ($isMerchantRealName) {
                array_push($div_members, ['member_id' => $commissionPreData['merchant_member_id'], 'amount' => sprintf("%.2f", $commissionPreData['merchant_amount']), 'fee_flag' => 'N']);
            }
        }

        //平台本身
        $aimeiyuePrice = $total_amount - $commissionPreData['merchant_amount'];
        self::addRecord($sp_ogid, $order_id, $orderObj->goods_amount, $aimeiyuePrice, '', 0, $buyer_id, 0, 1, $payconfirm_no);
        array_push($div_members, ['member_id' => 0, 'amount' => sprintf("%.2f", $aimeiyuePrice), 'fee_flag' => 'Y']);

        //确认分佣金额
        if (!$isMerchantRealName) {
            $commissionPreData['merchant_amount'] = 0;
        }
        $confirm_amt = $aimeiyuePrice + $commissionPreData['merchant_amount'];
        return ['div_members' => $div_members, 'confirm_amt' => sprintf("%.2f", $confirm_amt)];
    }

    /**
     * 新增分账记录
     * @param string $og_id 订单商品ID
     * @param int $order_id 订单ID
     * @param float $goods_amount 商品金额
     * @param float $divide_price 分账金额
     * @param string $commission 佣金比例
     * @param int $um_id 用户或商户ID
     * @param int $buyer_id 买家ID
     * @param int $sh_type 分账类型 推送类型 1:直推 2:间推 3:公司 4:手动分账 5:员工
     * @param int $isExistAccount 是否实名
     * @param string $payconfirm_no 支付确认编号
     */
    public static function addRecord($og_id, $order_id, $goods_amount, $divide_price, $commission = '', $um_id = 0, $buyer_id, $sh_type = 0, $isExistAccount, $payconfirm_no)
    {
        $recordObj = new self();
        $recordObj->order_id = $order_id;
        $recordObj->user_id = $buyer_id;
        $recordObj->og_id = trim($og_id, ',');
        $recordObj->order_price = $goods_amount;
        $recordObj->divide_price = $divide_price;
        $recordObj->proportion = trim($commission, ',');
        $recordObj->um_id = $um_id;
        $recordObj->sh_type = $sh_type;
        $recordObj->is_div = $isExistAccount ? 1 : 0; //是否分账
        $recordObj->payconfirm_no = $isExistAccount ? $payconfirm_no : '';
        $recordObj->save();
        Log::add('订单分佣记录', $recordObj->toArray());
    }

    public function order()
    {
        return $this->belongsTo(OrderInfo::class, 'order_id', 'id');
    }

    public function users()
    {
        return $this->belongsTo(User::class, 'user_id', 'id');
    }
}